@@ -13,16 +13,16 @@ class BW_GOHMM(BW):
 	def __init__(self):
 		super().__init__()
 
-	def fit(self, traces: Set, initial_model: GOHMM=None, nb_states: int=None,
+	def fit(self, traces, initial_model: GOHMM=None, nb_states: int=None,
 			random_initial_state: bool=False, output_file: str=None,
 			epsilon: float=0.01,  max_it: int= inf,
-			pp: str='', verbose: bool = True):
+			pp: str='', verbose: bool = True, return_data: bool= False):
 		"""
 		Fits the model according to ``traces``.
 
 		Parameters
 		----------
-		traces : list
+		traces : Set or list or numpy.ndarray
 			training set.
 		initial_model : GOHMM, optional.
 			first hypothesis. If not set it will create a random GOHMM with
@@ -53,12 +53,19 @@ class BW_GOHMM(BW):
 		verbose: bool, optional
 			Print or not a small recap at the end of the learning.
 			Default is True.
+		return_data: bool, optional
+			If set to True, a dictionary containing following values will be
+			returned alongside the hypothesis once the learning is done.
+			'learning_rounds', 'learning_time', 'training_set_loglikelihood'.
+			Default is False.
 
 		Returns
 		-------
 		GOHMM
 			fitted GOHMM.
 		"""
+		if type(traces) != Set:
+			traces = Set(traces, t=2)
 		if not initial_model:
 			if not nb_states:
 				print("Either nb_states or initial_model should be set")
